#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <net/dhcpelios/debug.h>
#include <network.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <net/ethernet.h>
#include <netinet/if_ether.h>
#include <net/if.h>
#include <net/if_dl.h>
#include <net/if_types.h>
#include "ecos.h"


//--------------------------------------------------------------------------------------------------
// strdup is not implemented with ecos
// Don't forget to free strdup strings ..
//--------------------------------------------------------------------------------------------------
char * strdup(const char *string)
{
 unsigned int len=strlen(string);
 char *newbit=(char *)malloc(len+1);
 DEBUG("strdup","Allocating %d for '%s'",len+1,string);
 if (!newbit) return(NULL);
 strncpy(newbit,string,len);
 newbit[len]=0;
 DEBUG("strdup","New :'%s'",newbit);
 return (newbit);
}

//--------------------------------------------------------------------------------------------
// Conversion d'une adresse IP exprime sous forme d'entier en une chaine notation dcimale pointe
//--------------------------------------------------------------------------------------------
char* DHCPNumber2String(const uint32_t addr,char* buffer)
{
 struct in_addr tmp;			// Socket

 tmp.s_addr=addr;
 strncpy(buffer,inet_ntoa(tmp),IPSTRING);
 return (buffer);
}

//--------------------------------------------------------------------------------------------
// Calcule la somme de controle INET (IP/UDP)
// sur un packet addr de longueur len
//--------------------------------------------------------------------------------------------
int DHCPinet_cksum(u_short *addr,const int len)
{
    register int nleft = len;
    register u_short *w = addr;
    register u_short answer;
    register u_int sum = 0;
    u_short odd_byte = 0;

    //
    //  Our algorithm is simple, using a 32 bit accumulator (sum),
    //  we add sequential 16 bit words to it, and at the end, fold
    //  back all the carry bits from the top 16 bits into the lower
    //  16 bits.
    //
    while( nleft > 1 )  {
        sum += *w++;
        nleft -= 2;
    }

    // mop up an odd byte, if necessary
    if( nleft == 1 ) {
        *(u_char *)(&odd_byte) = *(u_char *)w;
        sum += odd_byte;
    }

    //
    // add back carry outs from top 16 bits to low 16 bits
    //
    sum = (sum >> 16) + (sum & 0x0000ffff); // add hi 16 to low 16
    sum += (sum >> 16);                     // add carry
    answer = ~sum;                          // truncate to 16 bits
    return (answer);
}

//--------------------------------------------------------------------------------------------
// Envoie n paquets ICMP echo messages  adresse
// Retourne -1 en cas d'erreur
// Sinon retourne le nombre de paquets recus en rponse
//--------------------------------------------------------------------------------------------
int DHCPPingHost(const char adresse[IPSTRING],const unsigned int n)
{
    static unsigned char pkt1[256];  // Paquet envoye
    static unsigned char pkt2[256];  // Paquet recu

    struct icmp *icmp = (struct icmp *)pkt1;	// Paquet ICMP a envoyer
    int icmp_len = 64;          		// Taille minimale
    int seq=0;					// Numero du paquet
    unsigned int ok_recv=0;			// Nombre de paquets OK
    unsigned int bogus_recv=0;			// Nombre de paquet NOK
    cyg_tick_count_t *tp=NULL;			// Ticks horloge
    long *dp=NULL;				// Conversion des tics horloge
    struct sockaddr_in from;			// Descripteur de socket pour les rponses
    int i=0;					// Pour copie de la structure
    int len=0;
    int fromlen=0;
    struct protoent *p=NULL;		// Recherche dans la base des protocoles
    struct timeval tv;			// Temps de timeout
    struct sockaddr_in hot;		// Descripteur socket cible
    struct sockaddr_in* host=&hot;	// Pointeur vers descripteur socket cible
    int s=0;				// Descripteur de socket

    if ((p = getprotobyname("icmp")) == (struct protoent *)0)
    {
        LOG(LOG_ERR,"DHCPPingHost","Getprotobyname error");
        return(-1);
    }
    s = socket(AF_INET, SOCK_RAW, p->p_proto);
    if (s < 0)
    {
        LOG(LOG_ERR,"DHCPPingHost","Socket RAW error");
        return(-1);
    }
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    setsockopt(s, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv));

    // Set up host address
    host->sin_family=AF_INET;
    host->sin_len=sizeof(host);
    host->sin_addr.s_addr=inet_addr(adresse);
    host->sin_port=0;
    LOG(LOG_INFO,"DHCPPingHost","Pinging server %s", inet_ntoa(host->sin_addr));
    for (seq = 0;  seq < n;  seq++ )
    {
        // Build ICMP packet
        icmp->icmp_type = ICMP_ECHO;
        icmp->icmp_code = 0;
        icmp->icmp_cksum = 0;
        icmp->icmp_seq = seq;
        icmp->icmp_id = UNIQUEID;
        // Set up ping data
        tp = (cyg_tick_count_t *)&icmp->icmp_data;
        *tp++ = cyg_current_time();
        dp = (long *)tp;
	// Copie de la structure cyg_tick_count_t
        for (i = sizeof(*tp);  i < icmp_len;  i += sizeof(*dp))
	{
            *dp++ = i;
        }
        // Add checksum
        icmp->icmp_cksum = DHCPinet_cksum( (u_short *)icmp, icmp_len+8);
        // Send it off
        if (sendto(s, icmp, icmp_len+8, 0, (struct sockaddr *)host, sizeof(*host)) < 0) {
         LOG(LOG_ERR,"DHCPPingHost","Error during ICMP send :%s",strerror(errno));
         continue;
        }
        // Wait for a response
        fromlen = sizeof(from);
        len = recvfrom(s, pkt2, sizeof(pkt2), 0, (struct sockaddr *)&from, &fromlen);
        if (len < 0) {
            LOG(LOG_WARNING,"DHCPPingHost","Error during ICMP receiving :'%s'",strerror(errno));
	     bogus_recv++;
        } else {
            if (DHCPICMPtest(pkt2, len, &from, host)) {
                ok_recv++;
            } else {
                bogus_recv++;
            }
        }
    }
    close(s);
    LOG(LOG_INFO,"DHCPPingHost","Sent %d packets, received %d OK, %d bad", n, ok_recv, bogus_recv);
    return(ok_recv);
}

//--------------------------------------------------------------------------------------------
// Vrifie la validit d'un paquet ICMP echo reply (pkt) de longueur len
// Retourne 0 si invalide,1 sinon
//--------------------------------------------------------------------------------------------
char DHCPICMPtest(const unsigned char *pkt,const int len,const struct sockaddr_in *from,const struct sockaddr_in *to)
{
    int lon=len;
    cyg_tick_count_t *tp=NULL;
    cyg_tick_count_t tv;
    struct ip *ip=NULL;
    struct icmp *icmp=NULL;
    tv = cyg_current_time();
    ip = (struct ip *)pkt;


    if ((lon < sizeof(*ip)) || ip->ip_v != IPVERSION)
    {
        LOG(LOG_ERR,"DHCPICMPtest","%s: Short packet or not IP! - Len: %d, Version: %d",inet_ntoa(from->sin_addr), lon, ip->ip_v);
        return (0);
    }
    icmp = (struct icmp *)(pkt + sizeof(*ip));
    if (icmp->icmp_type != ICMP_ECHOREPLY) {
        LOG(LOG_ERR,"DHCPICMPtest","%s: Invalid ICMP - type: %d",inet_ntoa(from->sin_addr), icmp->icmp_type);
        return (0);
    }
    if (icmp->icmp_id != UNIQUEID) {
        LOG(LOG_ERR,"DHCPICMPtest","%s: ICMP received for wrong id - sent: %x, recvd: %x",inet_ntoa(from->sin_addr), UNIQUEID, icmp->icmp_id);
	return(0);
    }
    lon -= (sizeof(*ip) + 8);
    tp = (cyg_tick_count_t *)&icmp->icmp_data;

    LOG(LOG_INFO,"DHCPICMPtest","%d bytes from %s: ", lon, inet_ntoa(from->sin_addr));
    LOG(LOG_INFO,"DHCPICMPtest","icmp_seq=%d", icmp->icmp_seq);
    LOG(LOG_INFO,"DHCPICMPtest",", time=%d ms", (int)(tv - *tp)*10);
    return (from->sin_addr.s_addr == to->sin_addr.s_addr);
}

//--------------------------------------------------------------------------------------------
// Permet d'ajouter une adresse MAC dans la table ARP
// dest est l'adresse IP
// gateway l'adresse MAC
// index le numero de l'interface
// Note : la table de routage est limite a 54 entres
//--------------------------------------------------------------------------------------------
char DHCPAddMAC(const char* dest,const char* gateway,const int index)
{
 struct sockaddr_dl gway;		// Adresse passerelle
 struct sockaddr_in dst;		// Rseau/hote cible

 char ds[IPSTRING];
 struct ecos_rtentry *rt=NULL;		// Entre dans la table de routage
 int s=0;				// Descripteur de socket
 u_char cp[MACSTRING];			// Adresse MAC sous forme chaine

 memset(&gway,0,sizeof(struct sockaddr_dl));
 gway.sdl_family=AF_LINK; // AF_UNSPEC ?
 gway.sdl_len=sizeof(struct sockaddr_dl);
 gway.sdl_index=index;
 gway.sdl_type=IFT_ETHER;  // pas terrible !
 gway.sdl_alen=ETHER_ADDR_LEN;
 memcpy(gway.sdl_data,gateway,ETHER_ADDR_LEN);

 memset(&dst,0,sizeof(struct sockaddr_in));
 dst.sin_family=AF_INET;
 dst.sin_port=0;
 dst.sin_len=sizeof(struct sockaddr_in);
 dst.sin_addr.s_addr=inet_addr(dest);

 rt=(struct ecos_rtentry*)malloc(sizeof(struct ecos_rtentry));
 if (!rt)
 {
  LOG(LOG_ERR,"DHCPAddMAC","Error while allocating route :'%s'",strerror(errno));
  return(-1);
 }
 memset(rt,0,sizeof(struct ecos_rtentry));
 rt->rt_flags|=RTF_LLINFO;
 rt->rt_flags|=RTF_HOST; // Host entry
 memcpy(&(rt->rt_gateway), &gway, sizeof(struct sockaddr));
 memcpy(&(rt->rt_dst), &dst, sizeof(struct sockaddr_in));

 rt->rt_flags|=RTF_UP; // Route utilisable
// rt->rt_flags|=RTF_WAS_CLONED;
 //rt->rt_use=0;
 rt->rt_dev=NULL;
 rt->rt_metric=1;		// Reseau local
 strncpy(ds,inet_ntoa(dst.sin_addr),IPSTRING);
 DHCPether_print(gateway,cp,MACSTRING);
 LOG(LOG_INFO,"DHCPAddMAC","Adding MAC adress for '%s' :'%s' on interface eth%d",ds,cp,index-1);
 s=socket(AF_INET,SOCK_DGRAM,0);
 if (s<0)
 {
  LOG(LOG_ERR,"DHCPAddMAC","Socket error :'%s'",strerror(errno));
  free(rt);
  return(-1);
 }
 if (ioctl(s,SIOCADDRT,rt)<0)
 {
  LOG(LOG_ERR,"DHCPAddMAC","Ioctl error :'%s'",strerror(errno));
  switch (errno)
  {
   case EINVAL:LOG(LOG_ERR,"DHCPAddMAC","Invalid Command or agument");
	  break;
   case ENOTTY:LOG(LOG_ERR,"DHCPAddMAC","Not a device");
	  break;
   case EBADF:LOG(LOG_ERR,"DHCPAddMAC","Bad File descriptor");
	  break;
   case EEXIST:LOG(LOG_ERR,"DHCPAddMAC","Already existing");
	   break;
   default:LOG(LOG_ERR,"DHCPAddMAC","Unknown error %d:'%s'",errno,strerror(errno));
  }
  free(rt);
  close(s);
  return (-1);
 }
 LOG(LOG_INFO,"DHCPAddMAC","ARP entry added");
 free(rt);
 close(s);
 return(0);
}

//--------------------------------------------------------------------------------------------
// Permet de supprimer une adresse MAC de la table ARP
//--------------------------------------------------------------------------------------------
char DHCPDelMAC(const char* dest,const char* gateway,const int index)
{
 struct sockaddr_dl gway;			// Adresse passerelle
 struct sockaddr_in dst;			// Rseau/hote cible

 char ds[IPSTRING];
 struct ecos_rtentry *rt=NULL;		// Entre dans la table de routage
 int s=0;				// Descripteur de socket
 u_char cp[MACSTRING];

 memset(&gway,0,sizeof(struct sockaddr_dl));
 gway.sdl_family=AF_LINK; // AF_UNSPEC ?
 gway.sdl_len=sizeof(struct sockaddr_dl);
 gway.sdl_index=index;
 gway.sdl_type=IFT_ETHER;
 gway.sdl_alen=ETHER_ADDR_LEN;
 memcpy(gway.sdl_data,gateway,ETHER_ADDR_LEN);

 memset(&dst,0,sizeof(struct sockaddr_in));
 dst.sin_family=AF_INET;
 dst.sin_port=0;
 dst.sin_len=sizeof(struct sockaddr_in);
 dst.sin_addr.s_addr=inet_addr(dest);

 rt=(struct ecos_rtentry*)malloc(sizeof(struct ecos_rtentry));
 if (!rt)
 {
  LOG(LOG_ERR,"DHCPDelMAC","Error while allocating route :'%s'",strerror(errno));
  return(-1);
 }
 memset(rt,0,sizeof(struct ecos_rtentry));
 rt->rt_flags|=RTF_LLINFO;
 rt->rt_flags|=RTF_HOST; // Host entry
 memcpy(&(rt->rt_gateway), &gway, sizeof(struct sockaddr));
 memcpy(&(rt->rt_dst), &dst, sizeof(struct sockaddr_in));

 rt->rt_flags|=RTF_UP; // Route utilisable
 //rt->rt_use=0;
 rt->rt_dev=NULL;
 rt->rt_metric=1;		// Reseau local
 strncpy(ds,inet_ntoa(dst.sin_addr),IPSTRING);
 DHCPether_print(gateway,cp,MACSTRING);
 LOG(LOG_INFO,"DHCPDelMAC","Removing MAC adress for '%s' :'%s' on interface eth%d",ds,cp,index-1);
 s=socket(AF_INET,SOCK_DGRAM,0);
 if (s<0)
 {
  LOG(LOG_ERR,"DHCPDelMAC","Socket error %s",strerror(errno));
  free(rt);
  return(-1);
 }
 if (ioctl(s,SIOCDELRT,rt)<0)
 {
  LOG(LOG_ERR,"DHCPDelMAC","Ioctl error %s",strerror(errno));
  switch (errno)
  {
   case EINVAL:LOG(LOG_ERR,"DHCPDelMAC","Invalid Command or agument");
	  break;
   case ENOTTY:LOG(LOG_ERR,"DHCPDelMAC","Not a device");
	  break;
   case EBADF:LOG(LOG_ERR,"DHCPDelMAC","Bad File descriptor");
	  break;
   case EEXIST:LOG(LOG_ERR,"DHCPDelMAC","Already existing");
	   break;
   default:LOG(LOG_ERR,"DHCPDelMAC","Unknown error %d:%s",errno,strerror(errno));
  }
  free(rt);
  close(s);
  return (-1);
 }
 LOG(LOG_INFO,"DHCPDelMAC","ARP entry deleted");
 free(rt);
 close(s);
 return(0);
}

//--------------------------------------------------------------------------------------------
// Conversion d'une adresse MAC en chane de la forme "00:de:ad:00:be:ef"
// cp contient l'adresse sous forme binaire
// etheraddr est un buffer de longueur len octets destin  recevoir le rsultat.
//--------------------------------------------------------------------------------------------
char* DHCPether_print(const u_char cp[ETHER_ADDR_LEN], char *etheraddr,const unsigned int len)
{
 snprintf(etheraddr,len,"%02x:%02x:%02x:%02x:%02x:%02x",cp[0],cp[1],cp[2],cp[3],cp[4],cp[5]);
 return(etheraddr);
}
