/*

 * options.c -- DHCP server client/server options

 *

 * udhcp client/server

 * options.c -- DHCP server option packet tools

 * Rewrite by Russ Dill <Russ.Dill@asu.edu> July 2001

 *

 * Ported to eCOS by Sebastien Couret <sebastien.couret@elios-informatique.fr> November 2003

 *

 * This program is free software; you can redistribute it and/or modify

 * it under the terms of the GNU General Public License as published by

 * the Free Software Foundation; either version 2 of the License, or

 * (at your option) any later version.

 *

 * This program is distributed in the hope that it will be useful,

 * but WITHOUT ANY WARRANTY; without even the implied warranty of

 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

 * GNU General Public License for more details.

 *

 * You should have received a copy of the GNU General Public License

 * along with this program; if not, write to the Free Software

 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */



#ifndef _OPTIONS_H

#define _OPTIONS_H



#include "packet.h"



#define TYPE_MASK	0x0F



enum {

	OPTION_IP=1,

	OPTION_IP_PAIR,

	OPTION_STRING,

	OPTION_BOOLEAN,

	OPTION_U8,

	OPTION_U16,

	OPTION_S16,

	OPTION_U32,

	OPTION_S32

};



#define OPTION_REQ	0x10 // have the client request this option

#define OPTION_LIST	0x20 // There can be a list of 1 or more of these





struct dhcp_option {

	char name[10];

	char flags;

	unsigned char code;

};



extern struct dhcp_option options[];

extern int option_lengths[];



unsigned char *get_option(struct dhcpMessage *packet, int code);

int end_option(unsigned char *optionptr);

int add_option_string(unsigned char *optionptr, unsigned char *string);

int add_simple_option(unsigned char *optionptr, unsigned char code, u_int32_t data);

struct option_set *find_option(struct option_set *opt_list, char code);

void attach_option(struct option_set **opt_list, struct dhcp_option *option, char *buffer, int length);

void FreeOptionsList(struct option_set *opt_list);



#endif

