/*
 * packet.h -- DHCP server client/server packet creation
 *
 * udhcp client/server
 * Copyright (C) 1999 Matthew Ramsay <matthewr@moreton.com.au>
 *			Chris Trew <ctrew@moreton.com.au>
 *
 * Rewrite by Russ Dill <Russ.Dill@asu.edu> July 2001
 * Ported to eCOS by Sebastien Couret <sebastien.couret@elios-informatique.fr> November 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _PACKET_H
#define _PACKET_H

#include <netinet/udp.h>
#include <netinet/ip.h>
#define OPTSIZE 308	     // Option size

struct dhcpMessage {
	u_int8_t op;			//  Request or response : BOOTREQUEST,BOOTREPLY
	u_int8_t htype;			//  Hardware type : ETH_10M
	u_int8_t hlen;			//  Hardware adress length : ETH_10MB_LEN
	u_int8_t hops;
	u_int32_t xid;			// Transaction ID
	u_int16_t secs;
	u_int16_t flags;		// BROADCAST_FLAG
	u_int32_t ciaddr;		// IP adress of target
	u_int32_t yiaddr;
	u_int32_t siaddr;		// IP adress of server
	u_int32_t giaddr;		// IP adress of DHCP relay
	u_int8_t chaddr[16];		// MAC adress of target
	u_int8_t sname[64];		// Serveur name
	u_int8_t file[128];		// Boot file name
	u_int32_t cookie;		// Magic cookie : DHCP_MAGIC
	u_int8_t options[OPTSIZE];	// 312 - cookie
};

struct udp_dhcp_packet {
	struct udphdr udp;		// En tte UDP
	struct dhcpMessage data;	// Donnes DHCP
};

void init_header(struct dhcpMessage *packet, char type);
int get_packet(struct dhcpMessage *packet, int fd);
int raw_packet(struct dhcpMessage *payload, u_int32_t source_ip, int source_port,u_int32_t dest_ip, int dest_port, unsigned char *dest_arp, int ifindex);
int kernel_packet(struct dhcpMessage *payload, u_int32_t source_ip, int source_port,u_int32_t dest_ip, int dest_port);

#endif
